﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Ticktacktoe
{
    class Machine : IPlayer
    {

        private char side = 'x';
        private AI ai = null;
        private char [] oldState = null;

        public Machine(char side, AI ai)
        {
            this.side = side;
            this.ai = ai;
        }

        public char GetSide()
        {
            return this.side;
        }
        
        public void MakeStep(Game game)
        {
            List<int> free = game.GetFreeCells();
            Random rand = new Random();

            //случайным образом решаем, является ли текущий ход 
            //зондирующим (случайным) или жадным (максимально выгодным)
            if (rand.Next(0, 100) < 20) {
                //случайный ход
                //Console.WriteLine("random step");
                int _step = free[rand.Next(0, free.Count)];
                game.SetCell(_step, this.side);
                this.oldState = game.GetState(this.side);
                return;
            }

            //жадный ход
            Dictionary<int, double> rewards = new Dictionary<int, double>();
            foreach (int free_step in free)
            {
                //Console.WriteLine(free_step);
                //для каждого доступного хода оцениваем состояние игры после него
                Game newGame = (Game)game.Clone();
                newGame.SetCell(free_step, this.side);
                rewards[free_step] = this.ai.GetReward(newGame.GetState(this.side));
                //game.PrintField();
                //newGame.PrintField();
            }

            //выясняем, какое вознаграждение оказалось максимальным
            double maxReward = 0;
            int maxStep = 0;
            foreach (int _step in rewards.Keys)
            {
                double reward = rewards[_step];
                if (reward > maxReward)
                {
                    maxReward = reward;
                    maxStep = _step;
                }
            }

            //корректируем оценку прошлого состояния
            //с учетом ценности нового состояния
            if (this.oldState != null)
            {
                this.ai.Correct(this.oldState, maxReward);
            }
          
            game.SetCell(maxStep, this.side);
            //сохраняем текущее состояние для того, 
            //чтобы откорректировать её ценность на следующем ходе
            this.oldState = game.GetState(this.side);

        }

        public void Win()
        {
            if (oldState != null)
            {
                this.ai.Correct(this.oldState, 1);
            }
            Console.WriteLine();
            Console.WriteLine(this.side + " wins!");
            Console.WriteLine();
        }

        public void Lose()
        {
            if (oldState != null)
            {
                this.ai.Correct(this.oldState, 0);
            }
        }

        public void Draw()
        {
            if (oldState != null)
            {
                this.ai.Correct(this.oldState, 0.5);
            }
        }

        
    }
}
